/*
Copyright 2020 The Jenkins X Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
// Code generated by client-gen. DO NOT EDIT.

package v1

import (
	"context"
	"time"

	v1 "github.com/jenkins-x/jx-api/v4/pkg/apis/jenkins.io/v1"
	scheme "github.com/jenkins-x/jx-api/v4/pkg/client/clientset/versioned/scheme"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// PipelineActivitiesGetter has a method to return a PipelineActivityInterface.
// A group's client should implement this interface.
type PipelineActivitiesGetter interface {
	PipelineActivities(namespace string) PipelineActivityInterface
}

// PipelineActivityInterface has methods to work with PipelineActivity resources.
type PipelineActivityInterface interface {
	Create(ctx context.Context, pipelineActivity *v1.PipelineActivity, opts metav1.CreateOptions) (*v1.PipelineActivity, error)
	Update(ctx context.Context, pipelineActivity *v1.PipelineActivity, opts metav1.UpdateOptions) (*v1.PipelineActivity, error)
	Delete(ctx context.Context, name string, opts metav1.DeleteOptions) error
	DeleteCollection(ctx context.Context, opts metav1.DeleteOptions, listOpts metav1.ListOptions) error
	Get(ctx context.Context, name string, opts metav1.GetOptions) (*v1.PipelineActivity, error)
	List(ctx context.Context, opts metav1.ListOptions) (*v1.PipelineActivityList, error)
	Watch(ctx context.Context, opts metav1.ListOptions) (watch.Interface, error)
	Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts metav1.PatchOptions, subresources ...string) (result *v1.PipelineActivity, err error)
	PipelineActivityExpansion
}

// pipelineActivities implements PipelineActivityInterface
type pipelineActivities struct {
	client rest.Interface
	ns     string
}

// newPipelineActivities returns a PipelineActivities
func newPipelineActivities(c *JenkinsV1Client, namespace string) *pipelineActivities {
	return &pipelineActivities{
		client: c.RESTClient(),
		ns:     namespace,
	}
}

// Get takes name of the pipelineActivity, and returns the corresponding pipelineActivity object, and an error if there is any.
func (c *pipelineActivities) Get(ctx context.Context, name string, options metav1.GetOptions) (result *v1.PipelineActivity, err error) {
	result = &v1.PipelineActivity{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("pipelineactivities").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do(ctx).
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of PipelineActivities that match those selectors.
func (c *pipelineActivities) List(ctx context.Context, opts metav1.ListOptions) (result *v1.PipelineActivityList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1.PipelineActivityList{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("pipelineactivities").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do(ctx).
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested pipelineActivities.
func (c *pipelineActivities) Watch(ctx context.Context, opts metav1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Namespace(c.ns).
		Resource("pipelineactivities").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch(ctx)
}

// Create takes the representation of a pipelineActivity and creates it.  Returns the server's representation of the pipelineActivity, and an error, if there is any.
func (c *pipelineActivities) Create(ctx context.Context, pipelineActivity *v1.PipelineActivity, opts metav1.CreateOptions) (result *v1.PipelineActivity, err error) {
	result = &v1.PipelineActivity{}
	err = c.client.Post().
		Namespace(c.ns).
		Resource("pipelineactivities").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(pipelineActivity).
		Do(ctx).
		Into(result)
	return
}

// Update takes the representation of a pipelineActivity and updates it. Returns the server's representation of the pipelineActivity, and an error, if there is any.
func (c *pipelineActivities) Update(ctx context.Context, pipelineActivity *v1.PipelineActivity, opts metav1.UpdateOptions) (result *v1.PipelineActivity, err error) {
	result = &v1.PipelineActivity{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("pipelineactivities").
		Name(pipelineActivity.Name).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(pipelineActivity).
		Do(ctx).
		Into(result)
	return
}

// Delete takes name of the pipelineActivity and deletes it. Returns an error if one occurs.
func (c *pipelineActivities) Delete(ctx context.Context, name string, opts metav1.DeleteOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("pipelineactivities").
		Name(name).
		Body(&opts).
		Do(ctx).
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *pipelineActivities) DeleteCollection(ctx context.Context, opts metav1.DeleteOptions, listOpts metav1.ListOptions) error {
	var timeout time.Duration
	if listOpts.TimeoutSeconds != nil {
		timeout = time.Duration(*listOpts.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Namespace(c.ns).
		Resource("pipelineactivities").
		VersionedParams(&listOpts, scheme.ParameterCodec).
		Timeout(timeout).
		Body(&opts).
		Do(ctx).
		Error()
}

// Patch applies the patch and returns the patched pipelineActivity.
func (c *pipelineActivities) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts metav1.PatchOptions, subresources ...string) (result *v1.PipelineActivity, err error) {
	result = &v1.PipelineActivity{}
	err = c.client.Patch(pt).
		Namespace(c.ns).
		Resource("pipelineactivities").
		Name(name).
		SubResource(subresources...).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}
